/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.List;
import nl.cloudfarming.client.field.model.ActivityField;
import nl.cloudfarming.client.field.model.TreatmentZone;
import org.openide.nodes.ChildFactory;
import org.openide.nodes.Node;

/**
 * Provides the functionality for creating child nodes for
 * {@link ActivityField}s
 *
 * @author Wytse Visser
 */
public final class ActivityFieldChildFactory extends ChildFactory<TreatmentZone> implements PropertyChangeListener {

    private final ActivityField activityField;

    public ActivityFieldChildFactory(ActivityField activityField) {
        this.activityField = activityField;
        this.activityField.addPropertyChangeListener(this);
    }

    @Override
    protected boolean createKeys(List<TreatmentZone> toPopulate) {
        toPopulate.addAll(activityField.getTreatmentZones());
        return true;
    }

    @Override
    public Node createNodeForKey(TreatmentZone key) {
        return new TreatmentZoneNode(key);
    }

    @Override
    public void propertyChange(PropertyChangeEvent evt) {
        if (ActivityField.PROP_TREATMENTZONES.equals(evt.getPropertyName())) {
            refresh(true);
        }
    }
}
