/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type;

import java.awt.BorderLayout;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.openide.awt.UndoRedo;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.windows.TopComponent;

/**
 *
 * @author Timon Veenstra <monezz@gmail.com>
 */
@MultiViewElement.Registration(displayName = "#LBL_Field_DATA",
iconBase = "nl/cloudfarming/client/field/type/field.png",
mimeType = Field.MIMETYPE,
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "FieldData",
position = 1000)
@NbBundle.Messages({
    "LBL_Field_DATA=Data",
    "Label_field_data_panel=Field data"
})
public class FieldDataElement extends JPanel implements MultiViewElement {

    private FieldDataObject obj;
    private JToolBar toolbar = new JToolBar();
    private transient MultiViewElementCallback callback;
    private ASBodyPanel content;
    private Field field;
    private NameChangeListener nameChangeListener;

    public FieldDataElement(Lookup lkp) {
        obj = lkp.lookup(FieldDataObject.class);
        assert obj != null;
        this.field = obj.getLookup().lookup(Field.class);
        assert field != null;
        
        initComponents();
        setLayout(new BorderLayout());

        //FieldVisualPanel fieldVisualPanel = new FieldVisualPanel(obj.getLookup().lookup(Field.class));

        content = new ASBodyPanel(new FieldDataPanel(obj.getLookup().lookup(Field.class)));
        content.setIcon(ImageUtilities.loadImageIcon("nl/cloudfarming/client/field/type/field32.png", true));
        content.setTitle(Bundle.Label_field_data_panel());

        add(content);
    }

    @Override
    public String getName() {
        return "FieldDataElement";
    }

    /**
     * This method is called from within the constructor to initialize the form.
     */
    private void initComponents() {

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 400, Short.MAX_VALUE));
        layout.setVerticalGroup(
                layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING).addGap(0, 300, Short.MAX_VALUE));
    }

    @Override
    public JComponent getVisualRepresentation() {
        return this;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return obj.getLookup();
    }

    @Override
    public void componentOpened() {
        nameChangeListener = new NameChangeListener();
        field.addPropertyChangeListener(Field.PROP_NAME, nameChangeListener);
    }

    @Override
    public void componentClosed() {
        field.removePropertyChangeListener(Field.PROP_NAME, nameChangeListener);
    }

    @Override
    public void componentShowing() {
        if (callback != null) {
            updateName();
        }
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return UndoRedo.NONE;
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    private void updateName() {
        if (callback != null) {
            TopComponent tc = callback.getTopComponent();
            tc.setHtmlDisplayName(field.getName());
            tc.setDisplayName(field.getName());
            tc.setName(getName());
            tc.setToolTipText(getToolTipText());
        }
    }

    /**
     * Listens to changes on {@link Field#PROP_NAME}
     * Updates the name of the top component
     */
    private class NameChangeListener implements PropertyChangeListener {

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            if(pce.getPropertyName().equals(Field.PROP_NAME)){
                updateName();
            }
        }
        
    }
}
