/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.field.type.wizard;

import java.util.HashSet;
import java.util.List;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.field.model.TreatmentZone;
import nl.cloudfarming.client.geoviewer.Geometrical;
import nl.cloudfarming.client.util.swing.ASBodyPanel;
import org.netbeans.api.project.Project;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;

/**
 * 
 * @author Maciek
 */
public class GeometricalSelectController implements WizardDescriptor.Panel<WizardDescriptor>, ListSelectionListener {

    /**
     * The visual component that displays this panel. If you need to access the
     * component from this class, just use getComponent().
     */
    private final Set<ChangeListener> listeners = new HashSet<>(1);
    private ASBodyPanel component;
    private GeometricalSelectPanel visualPanel;
    private Class desiredClass;
    private String componentTitle;
    private WizardDescriptor descriptor;
    private int selectionMode;

    public GeometricalSelectController(Class desiredClass, String title, int selectionMode) {
        this.componentTitle = title;
        this.desiredClass = desiredClass;
        this.selectionMode = selectionMode;
        assert (desiredClass.equals(Field.class) || desiredClass.equals(TreatmentZone.class)) : "Only geometricals of type Field or Treatmentzone are currently supported by this panel!";
    }

    // Get the visual component for the panel. In this template, the component
    // is kept separate. This can be more efficient: if the wizard is created
    // but never displayed, or not all panels are displayed, it is better to
    // create only those which really need to be visible.
    @Override
    public ASBodyPanel getComponent() {
        if (component == null) {
            visualPanel = new GeometricalSelectPanel(desiredClass, selectionMode);
            visualPanel.getGeometricalsList().addListSelectionListener(this);
            visualPanel.getProjectList().addListSelectionListener(this);
            component = new ASBodyPanel(visualPanel);
            component.setName(visualPanel.getName());
            component.setTitle(componentTitle);
        }
        return component;
    }

    @Override
    public HelpCtx getHelp() {
        // Show no Help button for this panel:
        return HelpCtx.DEFAULT_HELP;
        // If you have context help:
        // return new HelpCtx("help.key.here");
    }

    @Override
    public boolean isValid() {
        if(visualPanel == null){
            return false;
        }
        if (visualPanel.getGeometricalsList().getSelectedValue() == null
                || visualPanel.getProjectList().getSelectedValue() == null) {
            return false;
        }
        return true;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.add(l);
        }
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        synchronized (listeners) {
            listeners.remove(l);
        }
    }
    
    protected final void fireChangeEvent() {
        Set<ChangeListener> ls;
        synchronized (listeners) {
            ls = new HashSet<>(listeners);
        }
        ChangeEvent ev = new ChangeEvent(this);
        for (ChangeListener l : ls) {
            l.stateChanged(ev);
        }
    }

    /**
     * Used to retrieve previous panel state. Not implemented for this panel.
     * @param wiz 
     */
    @Override
    public void readSettings(WizardDescriptor wiz) {
    }

    /**
     * Store selected Geometricals and selected project in the wizardDescriptor for retrievel after this wizard finishes.
     * @param wiz 
     */
    @Override
    public void storeSettings(WizardDescriptor wiz) {
        List<Geometrical> selectedGeometricals = visualPanel.getSelectedGeometricals();
        Project selectedProject = visualPanel.getSelectedProject();

        if (selectedGeometricals != null && selectedProject != null) {
            wiz.putProperty(GeometricalSelectUtil.PROP_SELECTED_GEOMETRICALS, selectedGeometricals);
            wiz.putProperty(GeometricalSelectUtil.PROP_SELECTED_PROJECT, selectedProject);
        }
    }

    @Override
    public void valueChanged(ListSelectionEvent e) {
        isValid();
        fireChangeEvent();
    }
}
